/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <unistd.h>
#include <sys/socket.h>
#include <glib.h>
#include <vpn_service.h>

#include "netconf.h"
#include "netstat.h"
#include "mptcp.h"
#include "network_state.h"
#include "vpn.h"
#include "wifi.h"

using ::testing::InitGoogleTest;
using ::testing::Test;
using ::testing::TestCase;
using namespace std;

char *mesh_id = (char *)"gtestnetwork";

TEST(NetworkStatistics, GetWifiTotalTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.GetWifiTotalTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, GetWifiTotalRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.GetWifiTotalRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, GetWifiLastTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.GetWifiLastTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, GetWifiLastRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.GetWifiLastRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetCellularTotalTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetCellularTotalTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetCellularTotalRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetCellularTotalRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetCellularLastTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetCellularLastTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetCellularLastRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetCellularLastRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetWifiTotalTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetWifiTotalTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetWifiTotalRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetWifiTotalRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetWifiLastTxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetWifiLastTxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkStatistics, ResetWifiLastRxBytes_p)
{
	error_e ret = ERROR_NONE;
	NetStat mgr;

	ret = mgr.ResetWifiLastRxBytes();
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, AddRoute_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	const char *ip = "192.168.11.55";
	const char *mask = "255.255.255.0";
	const char *interface = "lo";
	const char *gateway = "192.168.11.1";
	gboolean result;

	ret = mgr.AddRoute(ip, mask, interface, gateway,
			AF_INET, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(true, result);
}

TEST(NetworkState, RemoveRoute_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	const char *ip = "192.168.11.55";
	const char *mask = "255.255.255.0";
	const char *interface = "lo";
	const char *gateway = "192.168.11.1";
	gboolean result;

	ret = mgr.RemoveRoute(ip, mask, interface, gateway,
			AF_INET, &result);

	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(true, result);
}

TEST(NetworkState, RemoveRoute_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean result;

	ret = mgr.RemoveRoute(NULL, NULL, NULL, NULL,
			-1, &result);

	EXPECT_NE(ERROR_NONE, ret);
}

TEST(NetworkState, LaunchMdns_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	const char *name = "gtest";

	ret = mgr.LaunchMdns(name);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, LaunchMdns_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	const char *name = "gtest";

	ret = mgr.LaunchMdns(name);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, DevicePolicySetWifi_p1)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state = 0;

	ret = mgr.DevicePolicySetWifi(state);
	EXPECT_EQ(ERROR_NONE, ret);
	ret = mgr.DevicePolicyGetWifi(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, 0);
}

TEST(NetworkState, DevicePolicySetWifi_p2)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state = 1;

	ret = mgr.DevicePolicySetWifi(state);
	EXPECT_EQ(ERROR_NONE, ret);
	ret = mgr.DevicePolicyGetWifi(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, 1);
}

TEST(NetworkState, DevicePolicyGetWifi_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state;

	ret = mgr.DevicePolicyGetWifi(&state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, DevicePolicyGetWifi_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;

	ret = mgr.DevicePolicyGetWifi(NULL);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, DevicePolicySetWifiProfile_p1)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state = 0;

	ret = mgr.DevicePolicySetWifiProfile(state);
	EXPECT_EQ(ERROR_NONE, ret);
	ret = mgr.DevicePolicyGetWifiProfile(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, 0);
}

TEST(NetworkState, DevicePolicySetWifiProfile_p2)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state = 1;

	ret = mgr.DevicePolicySetWifiProfile(state);
	EXPECT_EQ(ERROR_NONE, ret);
	ret = mgr.DevicePolicyGetWifiProfile(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, 1);
}

TEST(NetworkState, DevicePolicySetWifiProfile_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state = -1;

	ret = mgr.DevicePolicySetWifiProfile(state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, DevicePolicyGetWifiProfile_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	int state;

	ret = mgr.DevicePolicyGetWifiProfile(&state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, GetMeteredInfo_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.GetMeteredInfo(&state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, GetMeteredInfo_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.GetMeteredInfo(&state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, GetTCPDumpState_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.GetTCPDumpState(&state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, GetTCPDumpState_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;

	ret = mgr.GetTCPDumpState(NULL);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(NetworkState, StartTCPDump_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.StartTCPDump();
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(1);
	ret = mgr.GetTCPDumpState(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, true);
}

TEST(NetworkState, StartTCPDump_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	Wifi wifi_mgr;
	gboolean state;

	ret = mgr.StartTCPDump();
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(1);
	ret = mgr.GetTCPDumpState(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(state, false);

	wifi_mgr.LoadDriver(false);
}

TEST(NetworkState, StopTCPDump_p)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.StopTCPDump();
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(1);
	ret = mgr.GetTCPDumpState(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(state, false);
}

TEST(NetworkState, StopTCPDump_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	gboolean state;

	ret = mgr.StopTCPDump();
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(1);
	ret = mgr.GetTCPDumpState(&state);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(state, true);
}
TEST(NetworkState, PreferredIpv6Address_n)
{
	error_e ret = ERROR_NONE;
	NetworkState mgr;
	const char *profile = "gtest";
	char address[256];

	ret = mgr.PreferredIpv6Address(profile, address);
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_init_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "tun0";
	int iface_name_len = 4;
	int result;
	int h_index;
	char h_name[256];

	ret = mgr.vpn_init(iface_name, iface_name_len, &result, &h_index, h_name);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(result, VPNSVC_ERROR_NONE);
}

TEST(VpnTest, vpn_init_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "gtest";
	int iface_name_len = 5;
	int result;
	int h_index;
	char h_name[256];

	ret = mgr.vpn_init(iface_name, iface_name_len, &result, &h_index, h_name);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(result, false);
}

TEST(VpnTest, vpn_deinit_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *dev_name = "tun0";
	int result;

	ret = mgr.vpn_deinit(dev_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_deinit_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *dev_name = "gtest";
	int result;

	ret = mgr.vpn_deinit(dev_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(result, false);
}

TEST(VpnTest, vpn_protect_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "tun0";
	int iface_name_len = 5;
	int result;
	int h_index;
	char h_name[256];

	ret = mgr.vpn_init(iface_name, iface_name_len, &result, &h_index, h_name);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_EQ(result, VPNSVC_ERROR_NONE);

	const char *dev_name = "tun0";

	ret = mgr.vpn_protect(dev_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_protect_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *dev_name = "gtest";
	int result;

	ret = mgr.vpn_deinit(dev_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_up_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	int result;

	ret = mgr.vpn_up(iface_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_up_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	int result;

	ret = mgr.vpn_up(iface_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_down_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	int result;

	ret = mgr.vpn_up(iface_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_down_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	int result;

	ret = mgr.vpn_up(iface_name, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_block_networks_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	int result;
	GVariantBuilder nets_builder;
	GVariant *nets_param_vpn;
	GVariant *nets_param_orig;

	g_variant_builder_init(&nets_builder, G_VARIANT_TYPE("a{si}"));
	g_variant_builder_add(&nets_builder, "{si}", "1.1.1.1", 24);
	nets_param_vpn = g_variant_builder_end(&nets_builder);

	g_variant_builder_init(&nets_builder, G_VARIANT_TYPE("a{si}"));
	g_variant_builder_add(&nets_builder, "{si}", "2.2.2.2", 24);
	nets_param_orig = g_variant_builder_end(&nets_builder);

	ret = mgr.vpn_block_networks(nets_param_vpn, 1, nets_param_orig, 1, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_unblock_networks_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	int result;

	ret = mgr.vpn_unblock_networks(&result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_update_settings_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	int iface_index = 1;
	const char *local_ip = "192.168.0.2";
	const char *remote_ip  = "192.168.0.3";
	int mtu = 10;
	int result;

	ret = mgr.vpn_update_settings(iface_index, local_ip,
			remote_ip, mtu, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_update_settings_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	int result;

	ret = mgr.vpn_update_settings(0, NULL, NULL, 0, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_add_route_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *route = "gtest";
	int prefix = 24;
	int result;

	ret = mgr.vpn_add_route(iface_name, route, prefix, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_add_route_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *route = "gtest";
	int prefix = 24;
	int result;

	ret = mgr.vpn_add_route(iface_name, route, prefix, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_remove_route_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *route = "gtest";
	int prefix = 24;
	int result;

	ret = mgr.vpn_add_route(iface_name, route, prefix, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_remove_route_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *route = "gtest";
	int prefix = 24;
	int result;

	ret = mgr.vpn_add_route(iface_name, route, prefix, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(VpnTest, vpn_add_dns_server_p)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *dns_server = "1.1.1.1";
	int result;

	ret = mgr.vpn_add_dns_server(iface_name, dns_server, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(VpnTest, vpn_add_dns_server_n)
{
	error_e ret = ERROR_NONE;
	Vpnsvc mgr;
	const char *iface_name = "wlan0";
	const char *dns_server = "1.1.1.1";
	int result;

	ret = mgr.vpn_add_dns_server(iface_name, dns_server, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(VPNSVC_ERROR_NONE, result);
}

TEST(WifiTest, GetWifiState_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	char state[256];

	ret = mgr.GetWifiState(state);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, IsIpConflictDetectEnabled_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	gboolean state;

	ret = mgr.IsIpConflictDetectEnabled(&state);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SetBgscan_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int scan_mode = 1;

	ret = mgr.SetBgscan(scan_mode);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetAutoscan_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	gboolean state;

	ret = mgr.GetAutoscan(&state);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetIpConflictState_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int state;

	ret = mgr.GetIpConflictState(&state);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetIpConflictPeriod_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int period;

	ret = mgr.GetIpConflictPeriod(&period);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetAutoscanmode_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int autoscanmode;

	ret = mgr.GetAutoscanmode(&autoscanmode);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, NetlinkScan_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *ssid = "gtest";

	ret = mgr.NetlinkScan(ssid);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, RequestWpsConnect_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *param = "PBC";

	ret = mgr.RequestWpsConnect(param);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetPasspoint_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int enable;

	ret = mgr.GetPasspoint(&enable);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SetPasspoint_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int enable = 0;

	ret = mgr.SetPasspoint(enable);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetConfigIds_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	ret = mgr.GetConfigIds();
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SaveConfiguration_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest";
	const char *name = "gtest";
	const char *ssid = "gtest";

	ret = mgr.SaveConfiguration(config_id, name, ssid, NULL);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, SaveEapConfiguration_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest";
	const char *name = "gtest";
	const char *ssid = "gtest";

	ret = mgr.SaveEapConfiguration(config_id, name, ssid);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SetIpConflictPeriod_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	unsigned int period = 25000;

	ret = mgr.SetIpConflictPeriod(period);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SetIpConflictPeriod_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	unsigned int period = 10;

	ret = mgr.SetIpConflictPeriod(period);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, LoadConfiguration_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest_managed_none";
	const char *name = "gtest";
	const char *ssid = "gtest";

	ret = mgr.SaveConfiguration(config_id, name, ssid, NULL);
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(2);

	ret = mgr.LoadConfiguration(config_id);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, LoadConfiguration_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "\0";

	ret = mgr.LoadConfiguration(config_id);
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(WifiTest, LoadEapConfiguration_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest_managed_ieee8021x";
	const char *name = "gtest";
	const char *ssid = "gtest";

	ret = mgr.SaveEapConfiguration(config_id, name, ssid);
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(2);

	ret = mgr.LoadEapConfiguration(config_id);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, LoadEapConfiguration_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "\0";

	ret = mgr.LoadEapConfiguration(config_id);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, RemoveConfiguration_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest";

	ret = mgr.RemoveConfiguration(config_id);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, RemoveConfiguration_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "\0";

	ret = mgr.RemoveConfiguration(config_id);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, SetConfigField_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "gtest";
	const char *key = "Identity";
	const char *value = "gtest";

	ret = mgr.SetConfigField(config_id, key, value);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, SetConfigField_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "\0";
	const char *key = "\0";
	const char *value = "\0";

	ret = mgr.SetConfigField(config_id, key, value);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, GetConfigPassphrase_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	const char *config_id = "gtest";
	const char *name = "gtest";
	const char *ssid = "gtest";
	const char *passphrase = "gtest";

	ret = mgr.SaveConfiguration(config_id, name, ssid, passphrase);
	EXPECT_EQ(ERROR_NONE, ret);

	char password[256];

	ret = mgr.GetConfigPassphrase(config_id, password);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, GetConfigPassphrase_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *config_id = "\0";
	char password[256];

	ret = mgr.GetConfigPassphrase(config_id, password);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, CreateEapConfig_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *profile_name =
		"/net/connman/service/wifi_e45d7553fd5b_gtest_managed_ieee8021x";

	ret = mgr.CreateEapConfig(profile_name);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, CreateEapConfig_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *profile_name = "/gtest";

	ret = mgr.CreateEapConfig(profile_name);
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(WifiTest, DeleteEapConfig_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *profile_name =
		"/net/connman/service/wifi_e45d7553fd5b_gtest_managed_ieee8021x";

	ret = mgr.CreateEapConfig(profile_name);
	EXPECT_EQ(ERROR_NONE, ret);

	ret = mgr.DeleteEapConfig(profile_name);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, IpConflictSetEnable_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	gboolean state;

	ret = mgr.IsIpConflictDetectEnabled(&state);
	EXPECT_EQ(ERROR_NONE, ret);

	ret = mgr.IpConflictSetEnable(!state);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, ReqSimAuth_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const unsigned char *rand_data =  (unsigned char *)"0123456789012345";
	unsigned int length = 16;
	gboolean result;

	ret = mgr.ReqSimAuth(rand_data, length, &result);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, ReqSimAuth_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	unsigned char rand[5] = {0, };
	const unsigned char *rand_data =  (unsigned char *)rand;
	unsigned int length = 5;
	gboolean result;

	ret = mgr.ReqSimAuth(rand_data, length, &result);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, ReqAkaAuth_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	unsigned char rand[] = {65, 61, 62, 63, 64, 65, 66, 67,
		68, 69, 70, 71, 72, 73, 74, 75};
	unsigned char auth[] = {65, 61, 62, 63, 64, 65, 66, 67,
		68, 69, 70, 71, 72, 73, 74, 75};
	const unsigned char *rand_data =  (unsigned char *)rand;
	const unsigned char *auth_data =  (unsigned char *)auth;
	int length = 16;
	gboolean result;

	ret = mgr.ReqAkaAuth(rand_data, length, auth_data, length, &result);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, ReqAkaAuth_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	unsigned char rand[5] = {0, };
	unsigned char auth[5] = {0, };
	const unsigned char *rand_data =  (unsigned char *)rand;
	const unsigned char *auth_data =  (unsigned char *)auth;
	int length = 5;
	gboolean result;

	ret = mgr.ReqAkaAuth(rand_data, length, auth_data, length, &result);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, CheckBlackList_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *name = "gtest";
	const char *sec_type = "gtest";
	const char *eap = "gtest";
	gboolean allowed;

	ret = mgr.CheckBlackList(name, sec_type, eap, &allowed);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, TdlsDisconnect_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "00:00:00:00:00:00";

	ret = mgr.TdlsDisconnect(peer_mac_address, &result);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, TdlsConnectedPeer_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	char peer_mac_address[256];

	ret = mgr.TdlsConnectedPeer(peer_mac_address);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, TdlsDiscover_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "\0";

	ret = mgr.TdlsDiscover(peer_mac_address, &result);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, TdlsChannelSwitch_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "00:00:00:00:00:00";
	int freq = 2600;

	ret = mgr.TdlsChannelSwitch(peer_mac_address, freq, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(result, 0);
}

TEST(WifiTest, TdlsChannelSwitch_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "\0";
	int freq = -1;

	ret = mgr.TdlsChannelSwitch(peer_mac_address, freq, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(result, 0);
}

TEST(WifiTest, TdlsCancelChannelSwitch_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "00:00:00:00:00:00";

	ret = mgr.TdlsCancelChannelSwitch(peer_mac_address, &result);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, TdlsCancelChannelSwitch_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int result;
	const char *peer_mac_address = "\0";

	ret = mgr.TdlsCancelChannelSwitch(peer_mac_address, &result);
	EXPECT_EQ(ERROR_NONE, ret);
	EXPECT_NE(result, 0);
}

TEST(WifiTest, AddVsie_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = 1;
	const char *vsie = "dd050016328000";

	ret = mgr.AddVsie(frame_id, vsie);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, AddVsie_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = -1;
	const char *vsie = "\0";

	ret = mgr.AddVsie(frame_id, vsie);
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(WifiTest, GetVsie_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = 1;
	char vsie[256];

	ret = mgr.GetVsie(frame_id, vsie);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, GetVsie_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = -1;
	char vsie[256];

	ret = mgr.GetVsie(frame_id, vsie);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, RemoveVsie_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = 1;
	const char *vsie = "dd050016328000";

	ret = mgr.RemoveVsie(frame_id, vsie);
	EXPECT_EQ(ERROR_NONE, ret);

}

TEST(WifiTest, RemoveVsie_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	int frame_id = -1;
	const char *vsie = "\0";

	ret = mgr.RemoveVsie(frame_id, vsie);
	EXPECT_NE(ERROR_NONE, ret);

}

TEST(WifiTest, EncryptPassphrase_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *passphrase = "gtest";
	char *enc_data = NULL;

	ret = mgr.EncryptPassphrase(&enc_data, passphrase);
	free(enc_data);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, EncryptPassphrase_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	const char *passphrase = "\0";
	char *enc_data = NULL;

	ret = mgr.EncryptPassphrase(&enc_data, passphrase);
	free(enc_data);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, DecryptPassphrase_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	const char *passphrase = "0123456789";
	char *enc_data = NULL;
	char *passphrase1 = NULL;

	ret = mgr.EncryptPassphrase(&enc_data, passphrase);
	EXPECT_EQ(ERROR_NONE, ret);
	ret = mgr.DecryptPassphrase(enc_data, &passphrase1);
	std::cerr << "decrypted password = " << passphrase1;
	std::cerr << " " << "expected = " << passphrase << endl;
	EXPECT_EQ(!strcmp(passphrase1, passphrase), true);

	free(enc_data);
	free(passphrase1);
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, DecryptPassphrase_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;
	char *passphrase = NULL;
	const char enc_data[] = "\0";

	ret = mgr.DecryptPassphrase(enc_data, &passphrase);
	free(passphrase);
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(WifiTest, Stop_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	ret = mgr.RemoveDriver();
	EXPECT_EQ(ERROR_NONE, ret);

	sleep(5);

	ret = mgr.Stop("gtest");
	EXPECT_NE(ERROR_NONE, ret);
}

TEST(WifiTest, Start_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	ret = mgr.Start("softap");
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, Stop_p)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	sleep(2);
	ret = mgr.Stop("softap");
	EXPECT_EQ(ERROR_NONE, ret);
}

TEST(WifiTest, Start_n)
{
	error_e ret = ERROR_NONE;
	Wifi mgr;

	ret = mgr.Start("gtest");
	EXPECT_NE(ERROR_NONE, ret);
}

int main(int argc, char **argv)
{
	int ret = -1;

	try {
		testing::InitGoogleTest(&argc, argv);
	} catch(...) {
		std::cout << "Exception occurred." << std::endl;
	}

	try {
		ret = RUN_ALL_TESTS();
	} catch (const ::testing::internal::GoogleTestFailureException& e) {
		ret = -1;
		std::cout << "GoogleTestFailureException was thrown:" << e.what() << std::endl;
	}

	return ret;
}
