/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __WIFI_H__
#define __WIFI_H__

#include "wifi.h"
#include "gdbus.h"

class Wifi:public GDbus {
public:
	Wifi();
	~Wifi();

error_e GetWifiState(char *state);
error_e IsIpConflictDetectEnabled(gboolean *state);
error_e SetBgscan(int scan_mode);
error_e GetAutoscan(gboolean *state);
error_e GetIpConflictState(int *state);
error_e GetIpConflictPeriod(int *period);
error_e GetAutoscanmode(int *autoscanmode);
error_e NetlinkScan(const char *ssid);
error_e RequestWpsConnect(const char *param);
error_e GetPasspoint(int *enable);
error_e SetPasspoint(int enable);
error_e GetConfigIds(void);
error_e SaveConfiguration(const char *config_id, const char *name,
		const char *ssid, const char *passphrase);
error_e SaveEapConfiguration(const char *config_id, const char *name,
		const char *ssid);
error_e SetIpConflictPeriod(unsigned int time);
error_e RemoveConfiguration(const char *config_id);
error_e LoadConfiguration(const char *config_id);
error_e LoadEapConfiguration(const char *config_id);
error_e SetConfigField(const char *config_id, const char *key,
		const char *value);
error_e GetConfigPassphrase(const char *config_id, char *password);
error_e CreateEapConfig(const char *profile_name);
error_e DeleteEapConfig(const char *profile_name);
error_e IpConflictSetEnable(bool detect);
error_e GetSimImsi(char *imsi);
error_e ReqSimAuth(const unsigned char *rand_data, unsigned int length,
	gboolean *result);
error_e ReqAkaAuth(const unsigned char *rand_data, int rand_length,
	const unsigned char *auth_data, int auth_length, gboolean *result);
error_e GetSimAuth(const unsigned char *auth_data, unsigned int length);
error_e CheckBlackList(const char *name, const char *sec_type,
		const char *eap, gboolean *allowed);
error_e TdlsDisconnect(const char *peer_mac_address, int *result);
error_e TdlsConnectedPeer(char *peer_mac_address);
error_e TdlsConnect(const char *peer_mac_address, int *result);
error_e TdlsDiscover(const char *peer_mac_address, int *result);
error_e TdlsChannelSwitch(const char *peer_mac_address, int freq, int *result);
error_e TdlsCancelChannelSwitch(const char *peer_mac_address, int *result);
error_e AddVsie(int frame_id, const char *vsie);
error_e GetVsie(int frame_id, char *vsie);
error_e RemoveVsie(int frame_id, const char *vsie);
error_e EncryptPassphrase(char **enc_data, const char *passphrase);
error_e DecryptPassphrase(const char *enc_data, char **passphrase);
error_e Stop(const char *device);
error_e Start(const char *device);
error_e LoadDriver(bool device_picker_test);
error_e RemoveDriver();
};


#endif /* __WIFI_H__ */
