/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __NETCONFIG_UNITTEST_H__
#define __NETCONFIG_UNITTEST_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>

#define GMAINTIMEOUT   20000
#define MAX_AP_LENGTH  33
#define MAX_KEY_LENGTH 64
#define MAX_PATH_LENGTH 256

#define FEATURE_WIFI       "http://tizen.org/feature/network.wifi"
#define FEATURE_WIFIDIRECT "http://tizen.org/feature/network.wifi.direct"
#define FEATURE_SOFTAP     "http://tizen.org/feature/network.wifi.softap"

#define WIFI_ADDRESS_PATH    "/sys/class/net/wlan0/address"
#define WIFI_P2P_PATH        "/sys/class/net/p2p0/address"
#define WIFI_P2P_CONFIG_PATH "/etc/wifi-direct/wifi-direct-manager.conf"
#define WFD_CONF_GROUP_NAME  "wfd-manager"

GMainLoop *g_pMainLoop;
guint g_nTimeoutId;
bool g_bFeatureWifi;
bool g_bFeatureP2P;
bool g_bFeatureSoftap;

#define RUN_GMAIN_LOOP(callback) {\
	g_pMainLoop = g_main_loop_new(NULL, false);\
	g_nTimeoutId = g_timeout_add(GMAINTIMEOUT, callback, g_pMainLoop);\
	g_main_loop_run(g_pMainLoop);\
	g_source_remove(g_nTimeoutId);\
	g_pMainLoop = NULL;\
}

#define QUIT_GMAIN_LOOP {\
	if (g_pMainLoop)\
		g_main_loop_quit(g_pMainLoop);\
}

#define SKIP_NOT_SUPPORTED(bFeature) {\
	if (!bFeature) {\
		printf("Feature is not support\n");\
		return;\
	} \
}

#ifdef __cplusplus
}
#endif

#endif /* __NETCONFIG_UNITTEST_H__ */
