/*
 * Network Configuration Module
 *
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __NETCONFIG_PLUGIN_H__
#define __NETCONFIG_PLUGIN_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include <gio/gio.h>
#include <glib-object.h>

#include "neterror.h"

#define NETCONFIG_ADD_FOUND_AP_NOTI		"add_found_ap_noti"
#define NETCONFIG_DEL_FOUND_AP_NOTI		"del_found_ap_noti"
#define NETCONFIG_ADD_PORTAL_NOTI		"add_portal_noti"
#define NETCONFIG_DEL_PORTAL_NOTI		"del_portal_noti"
#define NETCONFIG_ADD_IP_CONFLICT_NOTI		"add_ip_conflict_noti"
#define NETCONFIG_DEL_IP_CONFLICT_NOTI		"del_ip_conflict_noti"

struct wifi_authentication_data {
	int auth_result;
	int resp_length;
	int authentication_key_length;
	int cipher_length;
	int integrity_length;
	char *resp_data;
	char *authentication_key;
	char *cipher_data;
	char *integrity_data;
};

struct netconfig_headed_plugin_t {
	void(*pop_device_picker) (void);
	gboolean(*send_notification_to_net_popup) (const char *, const char *);
	int(*send_message_to_net_popup) (const char *, const char *, const char *, const char *);
	int(*send_restriction_to_net_popup) (const char *, const char *, const char *);
	void(*set_system_event) (int, int, int);
	void(*pop_wifi_connected_poppup) (const char *);
};

struct netconfig_telephony_plugin_t {
	void(*get_telephony_network_type) (int *svctype, int *pstype);
	gboolean(*wifi_get_sim_imsi) (void *wifi, GDBusMethodInvocation *context);
	netconfig_error_e(*wifi_req_aka_auth) (GArray *rand_data, GArray *autn_data,
			GDBusMethodInvocation *context, struct wifi_authentication_data **data);
	gboolean(*wifi_req_sim_auth) (GArray *rand_data,
			GDBusMethodInvocation *context, struct wifi_authentication_data **data);
	gboolean(*tapi_check_sim_state) (void);
	gboolean(*wifi_get_aka_authdata) (void *wifi,
			GDBusMethodInvocation *context, struct wifi_authentication_data **data);
	gboolean(*wifi_get_sim_authdata) (void *wifi,
			GDBusMethodInvocation *context, struct wifi_authentication_data **data);
};

struct netconfig_stc_plugin_t {
	void (*get_stc_dn_stats) (time_t, time_t, GSList **);
	void (*get_stc_wifi_stats) (time_t, time_t, GSList **);
};

#define BATTERY_ATM_LIST            "atm_list"
#define BATTERY_ATM_APP_ID          "atm_app_id"
#define BATTERY_ATM_RX              "atm_rx"
#define BATTERY_ATM_TX              "atm_tx"

#define BATTERY_DN_TIME_LEVEL_0     "time_level_0"
#define BATTERY_DN_TIME_LEVEL_1     "time_level_1"
#define BATTERY_DN_TIME_LEVEL_2     "time_level_2"
#define BATTERY_DN_TIME_LEVEL_3     "time_level_3"
#define BATTERY_DN_TIME_LEVEL_4     "time_level_4"
#define BATTERY_DN_TIME_LEVEL_5     "time_level_5"
#define BATTERY_DN_TIME_LEVEL_6     "time_level_6"

#define BATTERY_WIFI_TIME_LEVEL_0   "time_level_0"
#define BATTERY_WIFI_TIME_LEVEL_1   "time_level_1"
#define BATTERY_WIFI_TIME_LEVEL_2   "time_level_2"
#define BATTERY_WIFI_TIME_LEVEL_3   "time_level_3"
#define BATTERY_WIFI_TIME_LEVEL_4   "time_level_4"
#define BATTERY_WIFI_START_TIME     "start_time"
#define BATTERY_WIFI_END_TIME       "end_time"
#define BATTERY_WIFI_SCAN_TIME      "scan_time"

typedef struct {
	char *app_id;
	uint time;
	uint rx;
	uint tx;
} battery_app_data_s;

typedef struct {
	uint time_level_0;
	uint time_level_1;
	uint time_level_2;
	uint time_level_3;
	uint time_level_4;
	uint time_level_5;
	uint time_level_6;
	GSList *atm_list;
	GSList *data_list;
	time_t start_time;
	time_t end_time;
} battery_dn_data_s;

typedef struct {
	uint time_level_0;
	uint time_level_1;
	uint time_level_2;
	uint time_level_3;
	uint time_level_4;
	time_t start_time;
	time_t end_time;
	uint scan_time;
	GSList *atm_list;
	GSList *data_list;
} battery_wifi_data_s;

typedef void (*stc_get_stats_cb) (time_t from, time_t to, GSList **list);

struct netconfig_battery_plugin_t {
	void (*start_dn_data) (void);
	void (*end_dn_data) (int, stc_get_stats_cb);
	void (*update_dn_rssi) (int);

	void (*start_wifi_data) (void);
	void (*end_wifi_data) (int, stc_get_stats_cb);
	void (*update_wifi_scan) (int);
	void (*update_wifi_rssi) (int);

	void (*get_battery_dn_list) (void *, stc_get_stats_cb);
	void (*get_battery_wifi_list) (void *, stc_get_stats_cb);
};

typedef void *bm_data_h;
typedef enum {
	BM_DATA_TYPE_BLE,
	BM_DATA_TYPE_WIFI,
	BM_DATA_TYPE_CPU,
	BM_DATA_TYPE_DISPLAY,
	BM_DATA_TYPE_DEVICE_NETWORK,
	BM_DATA_TYPE_GPS_SENSOR,
	BM_DATA_TYPE_HRM_SENSOR,
	BM_DATA_TYPE_BATTERY,
	BM_DATA_TYPE_MAX
} bm_plugin_data_type_e;

struct netconfig_bm_plugin_t {
	int (*init) (void);
	int (*deinit) (void);
	int (*get_feature_data) (bm_data_h *, bm_plugin_data_type_e);
};

typedef enum {
	SYS_EVT_NETWORK_STATUS = 0,
	SYS_EVT_WIFI_STATE = 1,
	EKEY_NETWORK_STATUS = 10,
	EKEY_WIFI_STATE = 11,
	EVAL_NETWORK_WIFI = 20,
	EVAL_NETWORK_CELLULAR = 21,
	EVAL_NETWORK_ETHERNET = 22,
	EVAL_NETWORK_BT = 23,
	EVAL_NETWORK_DISCONNECTED = 24,
	EVAL_WIFI_CONNECTED = 25,
	EVAL_WIFI_ON = 26,
	EVAL_WIFI_OFF = 27,
} sys_evt_t;

void netconfig_complete_get_sim_imsi(void *wifi, GDBusMethodInvocation *context, char *imsi);
void netconfig_complete_get_aka_auth(void *wifi, GDBusMethodInvocation *context, GArray *array);
void netconfig_complete_get_sim_auth(void *wifi, GDBusMethodInvocation *context, GArray *array);
void netconfig_wifi_power_on();

#ifdef __cplusplus
}
#endif

#endif /* __NETCONFIG_PLUGIN_H__ */
