/*
 * Network Configuration Module
 *
 * Copyright (c) 2000 - 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __NETCONFIG_WIFIBACKGROUND_SCAN_H__
#define __NETCONFIG_WIFIBACKGROUND_SCAN_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "wifi.h"

#if defined TIZEN_WEARABLE
#define SCAN_PERIODIC_DELAY		15
#define SCAN_EXPONENTIAL_MIN	5
#define SCAN_EXPONENTIAL_MAX	320
#else
#define SCAN_PERIODIC_DELAY		10
#define SCAN_EXPONENTIAL_MIN	4
#define SCAN_EXPONENTIAL_MAX	128
#endif

enum {
	WIFI_BGSCAN_MODE_EXPONENTIAL = 0x00,
	WIFI_BGSCAN_MODE_PERIODIC,
	WIFI_BGSCAN_MODE_MAX,
};

void netconfig_wifi_bgscan_start(gboolean immediate_scan);
void netconfig_wifi_bgscan_stop(void);
gboolean netconfig_wifi_get_bgscan_state(void);
guint netconfig_wifi_bgscan_get_mode(void);

gboolean netconfig_wifi_get_scanning(void);
void netconfig_wifi_set_scanning(gboolean scanning);

void netconfig_wifi_set_bgscan_pause(gboolean pause);
gboolean netconfig_wifi_bgscan_set_interval(guint interval);
void netconfig_wifi_bgscan_get_interval(guint *interval);
gboolean netconfig_wifi_is_bgscan_paused(void);
void netconfig_wifi_set_bgscan_pause(gboolean pause);

gboolean handle_set_bgscan(Wifi *wifi, GDBusMethodInvocation *context, guint scan_mode);
gboolean handle_resume_bgscan(Wifi *wifi, GDBusMethodInvocation *context);
gboolean handle_pause_bgscan(Wifi *wifi, GDBusMethodInvocation *context);
gboolean handle_reset_bgscan_interval(Wifi *wifi, GDBusMethodInvocation *context);

gboolean handle_get_autoscan(Wifi *wifi, GDBusMethodInvocation *context);
gboolean handle_get_autoscanmode(Wifi *wifi, GDBusMethodInvocation *context);




#ifdef __cplusplus
}
#endif

#endif /* __NETCONFIG_WIFIBACKGROUND_SCAN_H__ */
